#!/bin/bash

usage() {
	echo "Usage: $0 <start|stop> {UUID}"
	exit 1
}
if [ $# -ne 2 ];  then
	usage
fi
if [ "$1" = "stop" ]; then
	# Stop old sessions?
	exit 0
fi 

# Make sure that GST_PLUGIN_PATH is exported
# Needed for backwards compatibility.  If this directory doesn't exist, then we are on ThinPro build 31.
if [ -d /usr/lib/hptc-rdp-enhancement/ ] ; then
	export GST_PLUGIN_PATH=/usr/lib/hptc-rdp-enhancement/gstreamer-0.10/
	export PATH=$PATH:/usr/lib/hptc-rdp-enhancement/bin
else
	export GST_PLUGIN_PATH=/usr/local/lib/gstreamer-0.10/
fi

# Escape certain characters
function bash_cleanup() {
	arg=$@
	#Adds a '\' in front of the character
	#${arg//pattern/string}
	arg=${arg//\\/\\\\} #backslash first
	arg=${arg//\~/\\\~}
	arg=${arg//\!/\\\!}
	arg=${arg//\$/\\\$}
	arg=${arg//\&/\\\&}
	arg=${arg//\(/\\\(}
	arg=${arg//\)/\\\)}
	arg=${arg//\`/\\\`}
	arg=${arg//\'/\\\'}
	arg=${arg//\ /\\\ }
	arg=${arg//\"/\\\"}
	echo $arg
}

### MAIN

# Set client command line args
ARGS=""

VIEW_CLIENT=/usr/bin/view_client
if [ ! -f $VIEW_CLIENT ]; then
	VIEW_CLIENT=/usr/bin/vmware-view
	ARGS="$ARGS --keep-wm-bindings"
fi
SESSION_ROOT="root/ConnectionType/view/connections/$2"
USBR_TYPE=$(mclient --quiet get root/USB/root/protocol)

SERVER_NAME="`mclient --quiet get $SESSION_ROOT/server`"
if [ "$SERVER_NAME" != "" ]; then
	ARGS="$ARGS --serverURL=$SERVER_NAME"
fi

USER_NAME="`mclient --quiet get $SESSION_ROOT/username`"
# Force blank username so the client doesn't pick up 'user'
ARGS="$ARGS --userName=$(bash_cleanup $USER_NAME)"

USER_PASSWORD="`mclient --quiet get $SESSION_ROOT/password`"
if [ "$USER_PASSWORD" != "" ]; then
	ARGS="$ARGS --password=$(bash_cleanup $USER_PASSWORD)"
fi

DOMAIN_NAME="`mclient --quiet get $SESSION_ROOT/domain`"
if [ "$DOMAIN_NAME" != "" ]; then
	ARGS="$ARGS --domainName=$DOMAIN_NAME"
fi

DESKTOP_NAME="`mclient --quiet get $SESSION_ROOT/desktop`"
if [ "$DESKTOP_NAME" != "" ]; then
	ARGS="$ARGS --desktopName=$(bash_cleanup $DESKTOP_NAME)"
fi

AUTOMATIC_LOGIN="`mclient --quiet get $SESSION_ROOT/automaticLogin`"
if [ "$AUTOMATIC_LOGIN" = "1" ]; then
	ARGS="$ARGS --nonInteractive"
fi

KEYBOARD="`mclient --quiet get root/keyboard/rdp_kb`"
if [ "$KEYBOARD" != "" ] ; then
	# Keyboard must be set with -k option directly in API client
	if [ -e "/usr/share/rdesktop/keymaps/$KEYBOARD" ] ; then
		if [ "${VIEW_CLIENT##*/}" = "view_client" ]; then
			ARGS="$ARGS --keyboardLayout $KEYBOARD"
		else
			RDPOPTS="-k $KEYBOARD"
		fi
	fi
fi

# Full rdesktop option forwarding via the registry is not supported by API client at this time
# Use /etc/view_client_rdp.opts to add RDP options to the view API client
if [ "${VIEW_CLIENT##*/}" = "vmware-view" ]; then
	RDPOPTS="$RDPOPTS $(mclient --quiet get root/ConnectionType/view/general/rdpOptions)"
	[ -n "$RDPOPTS" ] && ARGS="$ARGS --rdesktopOptions=\"$RDPOPTS\""
fi


MAPSOUND=`mclient --quiet get $SESSION_ROOT/mapSound`
if [ "$MAPSOUND" = "1" ] ; then
	MAPSOUNDKIND=`mclient --quiet get $SESSION_ROOT/mapSoundKind`
	if [ -n "$MAPSOUNDKIND" ]; then
		ARGS="$ARGS --redirect=sound:$MAPSOUNDKIND:alsa";
	fi
fi

SMARTCARD=`mclient --quiet get $SESSION_ROOT/smartcard`
LNK=/etc/vmware/smartcard.link
if [ "$SMARTCARD" = "1" ] ; then
	ARGS="$ARGS --redirect=scard";

	SMARTCARD_DIR=/usr/lib/pkcs11
	ln -s $SMARTCARD_DIR $LNK
else
	rm $LNK
fi

EnableUSBStorageMapping=`mclient --quiet get $SESSION_ROOT/EnableUSBStorageMapping`
if [ "$EnableUSBStorageMapping" = "1" ] && [ "$USBR_TYPE" = "local" ]; then
	MapUSBStorage=`mclient --quiet get $SESSION_ROOT/MapUSBStorage`
	ARGS="$ARGS --redirect=disk:$MapUSBStorage=/media"
fi

SERIAL=`mclient --quiet get $SESSION_ROOT/mapSerial`
if [ "$SERIAL" = "1" ] ; then
	ARGS="$ARGS --redirect=comport:COM1=/dev/ttyS0,COM2=/dev/ttyS1,COM3=/dev/ttyS2,COM4=/dev/ttyS3,COM5=/dev/ttyUSB0 --redirect=lptport:LPT1=/dev/lp0";
fi

# Add MMR if specified
if [ "`mclient --quiet get root/ConnectionType/rdesktop/general/enableMMR`" = "1" ] ; then
	ARGS="$ARGS --redirect=addin:HPRCtrl:mmrproxy:HPRCtrl --redirect=addin:HPWCtrl:mmrproxy:HPWCtrl --redirect=addin:HPRStrm:mmrproxy:HPRStrm --redirect=addin:HPWStrm:mmrproxy:HPWStrm "
fi

# Add USB Redirection if specified
if [ "`mclient --quiet get root/ConnectionType/rdesktop/general/enableUSBR`" = "1" ] ; then
	# Needed for backwards compatibility.  If this directory doesn't exist, then we are on ThinPro build 31.
	if [ ! -d /usr/lib/hptc-rdp-enhancements ] ; then
		ARGS="$ARGS --redirect=usbrdr"
	elif [ "`mclient --quiet get root/USB/root/protocol`" = "rdp" ] ; then
		ARGS="$ARGS --redirect=usbrdr"
	fi
fi


DATABASE=/etc/printcap
if [ "`mclient --quiet get $SESSION_ROOT/printerAutocreation`" = "1" ] ; then
    if [  -f $DATABASE ]; then
        while read line
        do
            if echo $line | grep '^#' > /dev/null 2>&1; then
                continue
            fi
            line=$(echo $line | cut -d: -f1)
            NAME=$(echo $line|cut -d\| -f1)
            # TODO: provide an approach to specify windows driver
            #WINDOWS_DRIVER=$(echo $line|cut -d\| -f2)
            # Do not forward printer via rdesktop, it will be forwarded via VMware View USBR
            if [ "$USBR_TYPE" = "view" ] && [ "$WINDOWS_DRIVER" = "usb" ]; then
                continue
            fi
            if [ -n "$NAME" ]; then
                ARGS="$ARGS --redirect=printer:$NAME"
                if [ -n "$WINDOWS_DRIVER" ]; then
                    ARGS="$ARGS=\"$WINDOWS_DRIVER\""
                fi
            fi
        done < $DATABASE
    fi
fi

case $1 in
	start)
		# Properly evaluate args with spaces such as:
		# --redirect=printer:Deskjet="hp deskject 845c"
		echo -e "$VIEW_CLIENT $ARGS";
		eval "$VIEW_CLIENT $ARGS"
		exit $?
		;;
	# Already handled stop at the start of the script
	*)
		usage
		;;
esac
